PROGRAM ServTime;

USES Dos;

CONST
  DayOKVer = 2;                { the day of the week is ok in version 2.12 }
  DayOKSub = 12;

TYPE
  ConnIdRec = RECORD           { attached server information }
    InUse       :BOOLEAN;
    ConnId      :BYTE;
    ServNet     :ARRAY [1..4] OF BYTE;
    ServAddr    :ARRAY [1..6] OF BYTE;
    ServSocket  :ARRAY [1..2] OF BYTE;
    RecvTimeOut :ARRAY [1..2] OF BYTE;
    RoutAddr    :ARRAY [1..6] OF BYTE;
    SeqNum      :BYTE;
    ConnNum     :BYTE;
    ConnOk      :BOOLEAN;
    MaxTimeOut  :ARRAY [1..2] OF BYTE;
    Filler      :ARRAY [1..5] OF BYTE
  END;

  FileServerNameRec = ARRAY [1..48] OF BYTE;

  ConnIdTbl = ARRAY [1..8] OF ConnIdRec;   { we can attach up to 8 servers }

  ConnIdTblPtr = ^ConnIdTbl;

  FileServerInfo = RECORD
    ServerName   :FileServerNameRec;
    NetWareVer   :BYTE;
    NetWareSub   :BYTE;
  END;

  XM_type = STRING[2];

CONST
  DayString :ARRAY [0..6] OF STRING[3]
             = ( 'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat' );
  MonthString :ARRAY [1..12] OF STRING[3]
              = ( 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
                  'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec' );

  FUNCTION GetBindryAccessLevel( VAR br, bw :BYTE ) :BOOLEAN;
  {
  GetDindryAccessLevel checks to see if IPX and NET3 are loaded and returns
  false if not.  Then is checks the workstation general access to the bindry
  and updates the read ( br ) and write ( bw ) arguments.
      Bindry Security Access Level
          0    Anyone
          1    Logged
          2    Object
          3    Supervisor
  NOTE: We know we are logged in if read and write are > 0
  }


  VAR
    r  :registers;
    Request  :RECORD
                length       :WORD;
                subfunction  :BYTE;
              END;
    Reply     :RECORD
                length       :WORD;
                access       :BYTE;
                ObjectId     :LONGINT;
              END;

  BEGIN
    r.ah := $E3;
    r.al := 1;
    r.ds := Seg( Request );
    r.si := Ofs( Request );
    r.es := Seg( Reply );
    r.di := Ofs( Reply );
    Request.length := 1;
    Request.subfunction := $46;
    FillChar( Reply, SizeOf( Reply ), 0 );
    Reply.length := 5;
    MsDos( r );
    IF ( r.al <> 0 ) OR ( Reply.ObjectId = 0 )
      THEN GetBindryAccessLevel := FALSE
      ELSE BEGIN
        GetBindryAccessLevel := TRUE;
        br := Reply.access AND $0f;
        bw := ( Reply.access shr 4 ) AND $0f;
      END
  END;

  FUNCTION FormatServerName( f :FileServerNameRec ) :STRING;
  {
  FormatServerName converts the ASCIIZ server name string to a pascal string
  }

  LABEL
    done;

  VAR
    i  :BYTE;
    s  :STRING;

  BEGIN
    s := '';
    FOR I := 1 TO 48 DO BEGIN
      IF f[i] = 0 THEN goto done;
      s := s + chr( f[i] );
    END;
  done:
    FormatServerName := s;
  END;

  FUNCTION CheckConsolePriv :BOOLEAN;
  {
  CheckConsole returns true if we have console privileges on the preferred
  server
  }

  VAR
    r         :registers;
    ReqBuff   :RECORD
                l :WORD;
                s :BYTE;
              END;
    ReplBuff  :WORD;

  BEGIN
    ReqBuff.l := 1;
    ReqBuff.s := $c8;
    ReplBuff := 0;
    r.ah := $E3;
    r.ds := Seg( ReqBuff );
    r.si := Ofs( ReqBuff );
    r.es := Seg( ReplBuff );
    r.di := Ofs( ReplBuff );
    MsDos( r );
    CheckConsolePriv := r.al = 0;
  END;

  PROCEDURE SetPreferredConnectionId( i :BYTE );
  {
  SetPreferredConnectionId makes the server with the connection id ( i ) the
  preferred server
  }

  VAR
    r  :registers;

  BEGIN
    r.ah := $F0;
    r.al := 00;
    r.dl := i;
    MsDos( r );
  END;

  FUNCTION GetConnectionIdTblAddr :ConnIdTblPtr;
  {
  GetConnectionIdTblAddr returns the address of the connection id table.
  The connection id table holds server information on up to 8 servers we
  are attached to.
  }

  VAR
    r  :registers;

  BEGIN
    r.ah := $EF;
    r.al := 3;
    MsDos( r );
    GetConnectionIdTblAddr := Ptr( r.es, r.si );
  END;

  FUNCTION GetFileServerInfo( VAR info :FileServerInfo ) :BOOLEAN;
  {
  GetFileServerInfo get the name and version of the preferred server.
  }

  VAR
    r  :registers;
    Request  :RECORD
                length       :WORD;
                subfunction  :BYTE;
              END;
    Reply     :RECORD
                length       :WORD;
                ServerName   :FileServerNameRec;
                NetWareVer   :BYTE;
                NetWareSub   :BYTE;
                filler       :ARRAY [1..89] OF BYTE;
              END;

  BEGIN
    r.ah := $E3;
    r.ds := Seg( Request );
    r.si := Ofs( Request );
    r.es := Seg( Reply );
    r.di := Ofs( Reply );
    Request.length := 1;
    Request.subfunction := $11;
    FillChar( Reply, SizeOf( Reply ), 0 );
    Reply.length := 139;
    MsDos( r );
    IF r.al <> 0
      THEN GetFileServerInfo := FALSE
      ELSE BEGIN
        GetFileServerInfo := TRUE;
        info.ServerName := Reply.ServerName;
        info.NetWareVer := Reply.NetWareVer;
        info.NetWareSub := Reply.NetWareSub;
      END
  END;

  PROCEDURE GetServerDateTime( VAR y, m, d, h, mi, s, dy :BYTE );
  {
  GetServerDateTime gets the date and time from the preferred server.
  }

  VAR
    r  :registers;
    Reply    :RECORD
                year, month, day, hour, min, sec, doy  :BYTE
              END;

  BEGIN
    r.ah := $E7;
    r.ds := Seg( Reply );
    r.dx := Ofs( Reply );
    MsDos( r );
    WITH Reply DO BEGIN
      y := year;
      m := month;
      d := day;
      h := hour;
      mi := min;
      s := sec;
      dy := doy
    END;
  END;


  PROCEDURE SetServerDateTime( y, m, d, h, mi, s :BYTE );
  {
  SetServerDateTime sets the date and time of the preferred server.
  }

  VAR
    r  :registers;
    Request  :RECORD
                length       :WORD;
                subfunction  :BYTE;
                year, month, day, hour, min, sec  :BYTE
              END;
    Reply     :WORD;

  BEGIN
    r.ah := $E3;
    r.ds := Seg( Request );
    r.si := Ofs( Request );
    r.es := Seg( Reply );
    r.di := Ofs( Reply );
    Request.length := 7;
    Request.subfunction := $ca;
    Request.year := y;
    Request.month := m;
    Request.day := d;
    Request.hour := h;
    Request.min := mi;
    Request.sec := s;
    Reply := 0;
    MsDos( r );
  END;

  FUNCTION FromNovellYear( ny :BYTE ) :INTEGER;
  {
  FromNovellYear returns the year from the Novell file server year
  The Novell year:  0 to 81 is 2000 to 2081
                   82 to 99 is 1982 to 1999
  }

  BEGIN
    IF ny < 82
      THEN FromNovellYear := ny + 2000
      ELSE FromNovellYear := ny + 1900;
  END;

  FUNCTION ToNovellYear( y :WORD ) :BYTE;
  {
  ToNovellYear returns the Novell year from the year
  }

  BEGIN
    IF y > 1999
      THEN ToNovellYear := y - 2000
      ELSE ToNovellYear := y - 1900;
  END;

  PROCEDURE AMorPM( VAR h :WORD; VAR x :XM_type );
  {
  AMorPM converts 24 hours ( h ) to 12 hours and return AM or PM
  }

  BEGIN
    IF h < 12
      THEN x := 'AM'
      ELSE x := 'PM';
    IF h > 12 THEN h := h - 12;
    IF h = 0 THEN h := 12;
  END;

  FUNCTION TimeString( h, m, s :WORD ) :STRING;
  {
  TimeString returns a formated string with the time provided by the
  arguments
  }

  VAR
    xm :XM_type;
    x  :WORD;
    t :STRING;
    q :STRING;

  BEGIN
    t := '';
    x := h;
    AMorPM( x, xm );
    IF x < 10 THEN t := t + ' ';
    Str( x, q );
    t := t + q + ':';
    IF m < 10 THEN t := t + '0';
    Str( m, q );
    t :=  t + q + ':';
    IF s < 10 THEN t := t + '0';
    Str( s, q );
    t := t + q + ' ' + xm;
    TimeString := t;
  END;

  FUNCTION CheckVersion( ver, sub, refVer, refSub :WORD ) :BOOLEAN;
  {
  CheckVersion returns true if ver.sub is >= refVer.refSub
  }

  BEGIN
    IF ver > refVer
      THEN CheckVersion := TRUE
      ELSE IF ver = refVer
             THEN IF sub >= refSub
                    THEN CheckVersion := TRUE
                    ELSE CheckVersion := FALSE
             ELSE CheckVersion := FALSE;
  END;


{
ServTime sets all attached servers ( that we have console privileges on )
date and time to the current workstation date and time.

    Check to see if IPX and NET3 are loaded.
    if not loaded then tell and abort
    Check to see if we are logged in.
    if not logged in then tell and abort
    Get the address of the connection id table.
    Tell the current workstation date and time.
    Loop through all 8 entries in the connection id table
        if the connection is active then
            set it as the preferred server
            get the file server name and version
            check the server version
            if version < 2.12
                then show date and time ( with no day )
                else show date and time ( with day )
            check console privileges
            if console privilege
                then get current date and time and set server date and time
                else tell no console privileges
}


VAR
  bread,
  bwrite    :BYTE;
  CT        :ConnIdTblPtr;
  i         :BYTE;
  LoggedIn  :BOOLEAN;
  sYear, sMonth, sDay, sHour, sMin, sSec, sWeekDay  :BYTE;
  cHour, cMin, cSec, cMSec :WORD;
  cDay, cMonth, cYear, cWeekDay  :WORD;
  info  :FileServerInfo;
  stat  :BOOLEAN;
  xm    :XM_type;
  showDay :BOOLEAN;
  day   :STRING[3];

BEGIN
  WriteLn;
  WriteLn( 'ServTime Version 1.04 by Thomas F. Wilkinson' );
  WriteLn( 'For more Novell utilities call PC-OHIO BBS at (216) 381-3320' );
  WriteLn;
  IF NOT  GetBindryAccessLevel( bread, bwrite ) THEN BEGIN
    WriteLn( 'ServTime requires Novell IPX and NET3 loaded.' );
    Exit;
  END;
  IF ( bread < 1 ) OR ( bwrite < 1 ) THEN BEGIN
    WriteLn( 'You must be logged in to use ServTime.' );
    Exit;
  END;
  CT := GetConnectionIdTblAddr;
  GetDate( cYear, cMonth, cDay, cWeekDay );
  GetTime( cHour, cMin, cSec, cMSec );
  Write( 'The current workstation date is ',
           DayString[ cWeekDay ], ' ',
           MonthString[ cMonth ], ' ', cDay:2, ', ',
           cYear );
  WriteLn( ' and time is ',
         TimeString( cHour, cMin, cSec ) );
  WriteLn;
  FOR i := 1 to 8 DO
    IF CT^[i].InUse
      THEN BEGIN
        SetPreferredConnectionId( i );
        stat := GetFileServerInfo( info );
        WITH info DO BEGIN
          showDay := CheckVersion( NetWareVer, NetWareSub,
                                   DayOKVer, DayOKSub );
          WriteLn( 'Server ', FormatServerName( ServerName ),
                   ' Version ', NetWareVer, '.', NetWareSub:2 );
        END;
        GetServerDateTime( sYear, sMonth, sDay, sHour, sMin, sSec,
                           sWeekDay );
        IF showDay THEN day := DayString[ sWeekDay ]
                   ELSE day := '';
        Write( '  Current ',
                 day, ' ',
                 MonthString[ sMonth ], ' ', sDay:2, ', ',
                 FromNovellYear( sYear ) );
        Write( ' ', TimeString( sHour, sMin, sSec ) );
        IF CheckConsolePriv
          THEN BEGIN
            GetDate( cYear, cMonth, cDay, cWeekDay );
            GetTime( cHour, cMin, cSec, cMSec );
            IF showDay THEN day := DayString[ cWeekDay ]
                       ELSE day := '';
            Write( ' setting to ',
                     day, ' ',
                     MonthString[ cMonth ], ' ', cDay:2, ', ',
                     cYear );
            WriteLn( ' ', TimeString( cHour, cMin, cSec ) );
            SetServerDateTime( ToNovellYear( cYear ),
                               cMonth, cDay, cHour, cMin, cSec );
          END
          ELSE WriteLn( ' No console privileges, time not set.' );
      END
END.